#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-17-05
#
# Check Point Identity Awareness status
#
# 2020-06-08: changed snmp-scan function
#
# snmpwalk sample
# .1.3.6.1.4.1.2620.1.38.25.1.1.1.0 = Counter32: 1
# .1.3.6.1.4.1.2620.1.38.25.1.1.2.0 = Counter32: 2
# .1.3.6.1.4.1.2620.1.38.25.1.1.3.0 = Counter32: 3
# .1.3.6.1.4.1.2620.1.38.25.1.1.4.0 = Counter32: 4
# .1.3.6.1.4.1.2620.1.38.25.1.1.5.0 = Counter32: 5
# .1.3.6.1.4.1.2620.1.38.25.1.2.1.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.38.25.1.2.2.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.38.25.1.2.3.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.38.25.1.2.4.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.38.25.1.2.5.0 = Counter32: 0
# .1.3.6.1.4.1.2620.1.38.25.1.3.1.0 = STRING: "doamin1.de"
# .1.3.6.1.4.1.2620.1.38.25.1.3.2.0 = STRING: "doamin1.de"
# .1.3.6.1.4.1.2620.1.38.25.1.3.3.0 = STRING: "domain1.de"
# .1.3.6.1.4.1.2620.1.38.25.1.3.4.0 = STRING: "domain2.de"
# .1.3.6.1.4.1.2620.1.38.25.1.3.5.0 = STRING: "domain2.de"
# .1.3.6.1.4.1.2620.1.38.25.1.4.1.0 = IpAddress: 10.1.1.13
# .1.3.6.1.4.1.2620.1.38.25.1.4.2.0 = IpAddress: 10.1.1.14
# .1.3.6.1.4.1.2620.1.38.25.1.4.3.0 = IpAddress: 10.1.1.31
# .1.3.6.1.4.1.2620.1.38.25.1.4.4.0 = IpAddress: 10.1.1.16
# .1.3.6.1.4.1.2620.1.38.25.1.4.5.0 = IpAddress: 10.1.1.17
# .1.3.6.1.4.1.2620.1.38.25.1.5.1.0 = Counter32: 77462
# .1.3.6.1.4.1.2620.1.38.25.1.5.2.0 = Counter32: 85917
# .1.3.6.1.4.1.2620.1.38.25.1.5.3.0 = Counter32: 58378
# .1.3.6.1.4.1.2620.1.38.25.1.5.4.0 = Counter32: 4810
# .1.3.6.1.4.1.2620.1.38.25.1.5.5.0 = Counter32: 6256
#
# sample info
# [[u'1', u'0', u'domain1.de', u'10.1.1.13', u'77462'],
#  [u'2', u'0', u'domain1.de', u'10.1.1.14', u'85917'],
#  [u'3', u'0', u'domain1.de', u'10.1.1.31', u'58378'],
#  [u'4', u'0', u'domain2.de', u'10.1.1.16', u'4810'],
#  [u'5', u'0', u'domain2.de', u'10.1.1.17', u'6256']]
#
import time
from dataclasses import dataclass
from typing import Optional, Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
    Result,
    State,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointIaDc:
    status_code: int
    status_str: str
    events: int


_dcstatus = {
    '0': 'Ok',
    '1': 'Bad Credentials',
    '2': 'Connectivity Error',
    '3': 'Internal Error',
    '4': 'Connection Time Out'
}


def parse_checkpoint_ia_adquery(string_table: StringTable) -> Optional[Dict[str, Dict[str, CheckpointIaDc]]]:
    domains = {}
    for entry in string_table:
        try:
            status, domain, ipaddr, events = entry
        except ValueError:
            return

        if len(ipaddr.split('.')) == 4:
            if not domains.get(domain):
                domains[domain] = {}

            domains[domain].update({
                ipaddr: CheckpointIaDc(
                    status_code=int(status),
                    status_str=_dcstatus.get(status, f'unknown: {status}'),
                    events=int(events)
                )
            })
    return domains


def discovery_checkpoint_ia_adquery(section: Dict[str, Dict[str, CheckpointIaDc]]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_ia_adquery(item, params, section: Dict[str, Dict[str, CheckpointIaDc]]) -> CheckResult:
    try:
        dcs = section[item]
    except KeyError:
        return

    events_sum = 0
    now_time = time.time()
    value_store = get_value_store()

    for key in dcs.keys():
        dc = dcs[key]
        events_sum += dc.events

        dc_ip_address = key.replace('.', '_')
        try:
            value = get_rate(value_store, f'checkpoint_ia_adquery_{dc_ip_address}', now_time, dc.events, raise_overflow=True)
        except GetRateError:
            value = 0
        yield from check_levels(
            value=value,
            label=f'DC {key} Events',
            metric_name=f'checkpoint_ia_adquery_{dc_ip_address}',
            render_func=lambda v: f'{v:.2f}/s',
            notice_only=True
        )

        if not dc.status_code == 0:
            yield Result(state=State.CRIT, notice=f'DC {key} connection status: {dc.status_str}')
        #else:
        #    yield Result(state=State.OK, notice=f'DC {key} connection status: {dc.status_str}')

    yield Result(state=State.OK, summary=f'# of DCs: {len(dcs)}', details=' ')

    try:
        value = get_rate(value_store, f'checkpoint_ia_adquery_events_sum', now_time, events_sum, raise_overflow=True)
    except GetRateError:
        value = 0
    yield from check_levels(
        value=value,
        label='Events (sum)',
        metric_name=f'checkpoint_ia_adquery_events_sum',
        render_func=lambda v: f'{v:.2f}/s',
    )


register.snmp_section(
    name='checkpoint_ia_adquery',
    parse_function=parse_checkpoint_ia_adquery,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.38.25.1',  # CHECKPOINT-MIB::identityAwarenessADQueryStatusEntry
        oids=[
            '2',  # identityAwarenessADQueryStatusCurrStatus
            '3',  # identityAwarenessADQueryStatusDomainName
            '4',  # identityAwarenessADQueryStatusDomainIP
            '5',  # identityAwarenessADQueryStatusEvents
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_ia_adquery',
    service_name='Identity Awareness AD Domain %s',
    discovery_function=discovery_checkpoint_ia_adquery,
    check_function=check_checkpoint_ia_adquery,
    check_default_parameters={
    },
    check_ruleset_name='checkpoint_ia_adquery',
)
